@@ -218,13 +218,15 @@ Call this when starting a new session to ensure no stale state persists."
         pi-coding-agent--assistant-header-shown nil
         pi-coding-agent--local-user-message nil
         pi-coding-agent--extension-status nil
+        pi-coding-agent--working-message nil
         pi-coding-agent--in-code-block nil
         pi-coding-agent--in-thinking-block nil
         pi-coding-agent--thinking-marker nil
         pi-coding-agent--thinking-start-marker nil
         pi-coding-agent--thinking-raw nil
         pi-coding-agent--line-parse-state 'line-start
-        pi-coding-agent--pending-tool-overlay nil)
+        pi-coding-agent--pending-tool-overlay nil
+        pi-coding-agent--activity-phase "idle")
   ;; Use accessors for cross-module state
   (pi-coding-agent--set-last-usage nil)
   (pi-coding-agent--clear-followup-queue)
@@ -469,13 +471,17 @@ The name is displayed in the resume picker and header-line."
          (input (or (plist-get tokens :input) 0))
          (output (or (plist-get tokens :output) 0))
          (total (or (plist-get tokens :total) 0))
+         (cache-read (or (plist-get tokens :cacheRead) 0))
+         (cache-write (or (plist-get tokens :cacheWrite) 0))
          (cost (or (plist-get stats :cost) 0))
          (messages (or (plist-get stats :userMessages) 0))
          (tools (or (plist-get stats :toolCalls) 0)))
-    (format "Tokens: %s in / %s out (%s total) | Cost: $%.2f | Messages: %d | Tools: %d"
+    (format "Tokens: %s in / %s out (%s total) | Cache: R%s / W%s | Cost: $%.2f | Messages: %d | Tools: %d"
             (pi-coding-agent--format-number input)
             (pi-coding-agent--format-number output)
             (pi-coding-agent--format-number total)
+            (pi-coding-agent--format-number cache-read)
+            (pi-coding-agent--format-number cache-write)
             cost messages tools)))
 
 (defun pi-coding-agent-session-stats ()
@@ -519,14 +525,16 @@ Optional CUSTOM-INSTRUCTIONS provide guidance for the compaction summary."
   (when-let ((proc (pi-coding-agent--get-process))
              (chat-buf (pi-coding-agent--get-chat-buffer)))
     (message "Pi: Compacting...")
-    (pi-coding-agent--spinner-start)
+    (with-current-buffer chat-buf
+      (pi-coding-agent--set-activity-phase "compact"))
     (pi-coding-agent--rpc-async proc
                    (if custom-instructions
                        (list :type "compact" :customInstructions custom-instructions)
                      '(:type "compact"))
                    (lambda (response)
-                     ;; Pass chat-buf explicitly (callback may run in arbitrary context)
-                     (pi-coding-agent--spinner-stop chat-buf)
+                     (when (buffer-live-p chat-buf)
+                       (with-current-buffer chat-buf
+                         (pi-coding-agent--set-activity-phase "idle")))
                      (if (plist-get response :success)
                          (when (buffer-live-p chat-buf)
                            (with-current-buffer chat-buf
